import logging
from contextlib import contextmanager

import elasticsearch
import elasticsearch.helpers
from pyramid.settings import aslist, asbool


logger = logging.getLogger(__name__)


class Indexer(object):
    def __init__(self, hosts, prefix="kinto", force_refresh=False):
        self.client = elasticsearch.Elasticsearch(hosts)
        self.prefix = prefix
        self.force_refresh = force_refresh

    def indexname(self, bucket_id, collection_id):
        return "{}-{}-{}".format(self.prefix, bucket_id, collection_id)

    def create_index(self, bucket_id, collection_id, schema=None):
        indexname = self.indexname(bucket_id, collection_id)
        # Only if necessary.
        if not self.client.indices.exists(index=indexname):
            if schema:
                body = {"mappings": {indexname: schema}}
            else:
                body = None
            return self.client.indices.create(index=indexname, body=body)
        else:
            return self.update_index(bucket_id, collection_id, schema)

    def update_index(self, bucket_id, collection_id, schema=None):
        indexname = self.indexname(bucket_id, collection_id)
        if schema is None:
            schema = {"properties": {}}
        self.client.indices.put_mapping(index=indexname,
                                        doc_type=indexname,
                                        body=schema)

    def delete_index(self, bucket_id, collection_id=None):
        if collection_id is None:
            collection_id = "*"
        indexname = self.indexname(bucket_id, collection_id)
        try:
            return self.client.indices.delete(index=indexname)
        except elasticsearch.exceptions.NotFoundError:  # pragma: no cover
            pass

    def search(self, bucket_id, collection_id, **kwargs):
        indexname = self.indexname(bucket_id, collection_id)
        return self.client.search(index=indexname,
                                  doc_type=indexname,
                                  **kwargs)

    def flush(self):
        self.client.indices.delete(index="{}-*".format(self.prefix))

    @contextmanager
    def bulk(self):
        bulk = BulkClient(self)
        yield bulk
        elasticsearch.helpers.bulk(self.client,
                                   bulk.operations,
                                   refresh=self.force_refresh)


class BulkClient:
    def __init__(self, indexer):
        self.indexer = indexer
        self.operations = []

    def index_record(self, bucket_id, collection_id, record, id_field):
        indexname = self.indexer.indexname(bucket_id, collection_id)
        record_id = record[id_field]
        self.operations.append({
            '_op_type': 'index',
            '_index': indexname,
            '_type': indexname,
            '_id': record_id,
            '_source': record,
        })

    def unindex_record(self, bucket_id, collection_id, record, id_field):
        indexname = self.indexer.indexname(bucket_id, collection_id)
        record_id = record[id_field]
        self.operations.append({
            '_op_type': 'delete',
            '_index': indexname,
            '_type': indexname,
            '_id': record_id,
        })


def heartbeat(request):
    """Test that ElasticSearch is operationnal.

    :param request: current request object
    :type request: :class:`~pyramid:pyramid.request.Request`
    :returns: ``True`` is everything is ok, ``False`` otherwise.
    :rtype: bool
    """
    indexer = request.registry.indexer
    try:
        return indexer.client.ping()
    except Exception as e:
        logger.exception(e)
        return False


def load_from_config(config):
    settings = config.get_settings()
    hosts = aslist(settings.get('elasticsearch.hosts', 'localhost:9200'))
    prefix = settings.get('elasticsearch.index_prefix', 'kinto')
    force_refresh = asbool(settings.get('elasticsearch.force_refresh', 'false'))
    indexer = Indexer(hosts=hosts, prefix=prefix, force_refresh=force_refresh)
    return indexer



# -*- coding: utf-8 -*-
import os
import configparser

from kinto import main as kinto_main
from kinto.core.testing import get_user_headers, BaseWebTest as CoreWebTest


here = os.path.abspath(os.path.dirname(__file__))


class BaseWebTest(CoreWebTest):
    api_prefix = "v1"
    entry_point = kinto_main
    config = 'config.ini'

    def __init__(self, *args, **kwargs):
        super(BaseWebTest, self).__init__(*args, **kwargs)
        self.headers.update(get_user_headers('mat'))

    def tearDown(self):
        super().tearDown()
        self.app.app.registry.indexer.flush()

    @classmethod
    def get_app_settings(cls, extras=None):
        ini_path = os.path.join(here, cls.config)
        config = configparser.ConfigParser()
        config.read(ini_path)
        settings = dict(config.items('app:main'))
        if extras:
            settings.update(extras)
        return settings



import copy
import mock
import time
import unittest

import elasticsearch
from kinto.core import utils as core_utils
from kinto.core.testing import get_user_headers

from kinto_elasticsearch import __version__ as elasticsearch_version
from . import BaseWebTest


class PluginSetup(BaseWebTest, unittest.TestCase):

    def test_elasticsearch_capability_exposed(self):
        resp = self.app.get('/')
        capabilities = resp.json['capabilities']
        self.assertIn('elasticsearch', capabilities)
        expected = {
            "version": elasticsearch_version,
            "description": "Index and search records using ElasticSearch.",
            "url": "https://github.com/Kinto/kinto-elasticsearch"
        }
        assert expected == capabilities['elasticsearch']

    def test_indexer_flush(self):
        with mock.patch("kinto_elasticsearch.indexer.Indexer.flush") as flush:
            self.app.post("/__flush__", status=202)
            assert flush.called

    def test_present_in_heartbeat(self):
        resp = self.app.get("/__heartbeat__")
        assert "elasticsearch" in resp.json

    def test_returns_false_if_connection_fails(self):
        with mock.patch("kinto_elasticsearch.indexer.elasticsearch.Elasticsearch.ping",
                        side_effect=ValueError):
            resp = self.app.get("/__heartbeat__", status=503)
            assert not resp.json["elasticsearch"]


class PostActivation(BaseWebTest, unittest.TestCase):

    @classmethod
    def get_app_settings(cls, extras=None):
        settings = super().get_app_settings(extras)
        settings['storage_backend'] = 'kinto.core.storage.postgresql'
        settings['storage_url'] = 'postgres://postgres:postgres@localhost:5432/postgres'
        settings['permission_backend'] = 'kinto.core.permission.postgresql'
        settings['permission_url'] = settings['storage_url']
        return settings

    def setUp(self):
        app = self.make_app(settings={"kinto.includes": ""})
        capabilities = app.get("/").json["capabilities"]
        assert "elasticsearch" not in capabilities

        app.put("/buckets/bid", headers=self.headers)
        app.put("/buckets/bid/collections/cid", headers=self.headers)
        app.post_json("/buckets/bid/collections/cid/records",
                      {"data": {"before": "indexing"}},
                      headers=self.headers)

    def test_search_does_not_fail(self):
        resp = self.app.get("/buckets/bid/collections/cid/records",
                            headers=self.headers)
        assert len(resp.json["data"]) == 1

        resp = self.app.get("/buckets/bid/collections/cid/search",
                            headers=self.headers)
        results = resp.json
        assert len(results["hits"]["hits"]) == 0

    def test_record_creation_does_not_fail(self):
        self.app.post_json("/buckets/bid/collections/cid/records",
                           {"data": {"after": "indexing"}},
                           headers=self.headers)

        resp = self.app.get("/buckets/bid/collections/cid/search",
                            headers=self.headers)
        results = resp.json
        assert len(results["hits"]["hits"]) == 1
        assert results["hits"]["hits"][0]["_source"]["after"] == "indexing"


class RecordIndexing(BaseWebTest, unittest.TestCase):

    def setUp(self):
        self.app.put("/buckets/bid", headers=self.headers)
        self.app.put("/buckets/bid/collections/cid", headers=self.headers)
        resp = self.app.post_json("/buckets/bid/collections/cid/records",
                                  {"data": {"hello": "world"}},
                                  headers=self.headers)
        self.record = resp.json["data"]

    def test_new_records_are_indexed(self):
        resp = self.app.post("/buckets/bid/collections/cid/search",
                             headers=self.headers)
        result = resp.json
        assert result["hits"]["hits"][0]["_source"] == self.record

    def test_deleted_records_are_unindexed(self):
        rid = self.record["id"]
        self.app.delete("/buckets/bid/collections/cid/records/{}".format(rid),
                        headers=self.headers)
        resp = self.app.post("/buckets/bid/collections/cid/search",
                             headers=self.headers)
        result = resp.json
        assert len(result["hits"]["hits"]) == 0

    def test_response_is_served_if_indexer_fails(self):
        with mock.patch("kinto_elasticsearch.indexer.elasticsearch.helpers.bulk",
                        side_effect=elasticsearch.ElasticsearchException):
            r = self.app.post_json("/buckets/bid/collections/cid/records",
                                   {"data": {"hola": "mundo"}},
                                   headers=self.headers)
            assert r.status_code == 201


class ParentDeletion(BaseWebTest, unittest.TestCase):

    def setUp(self):
        self.app.put("/buckets/bid", headers=self.headers)
        self.app.put("/buckets/bid/collections/cid", headers=self.headers)
        resp = self.app.post_json("/buckets/bid/collections/cid/records",
                                  {"data": {"hello": "world"}},
                                  headers=self.headers)

    def index_exists(self, bucket_id, collection_id):
        indexer = self.app.app.registry.indexer
        indexname = indexer.indexname(bucket_id, collection_id)
        return indexer.client.indices.exists(indexname)

    def test_index_is_deleted_when_collection_is_deleted(self):
        self.app.delete("/buckets/bid/collections/cid", headers=self.headers)
        assert not self.index_exists("bid", "cid")

    def test_index_is_deleted_when_bucket_is_deleted(self):
        self.app.delete("/buckets/bid", headers=self.headers)
        assert not self.index_exists("bid", "cid")


class SearchView(BaseWebTest, unittest.TestCase):
    def setUp(self):
        self.app.put("/buckets/bid", headers=self.headers)
        self.app.put("/buckets/bid/collections/cid", headers=self.headers)

    def test_search_response_is_empty_if_indexer_fails(self):
        with mock.patch("kinto_elasticsearch.indexer.Indexer.search",
                        side_effect=elasticsearch.ElasticsearchException):
            resp = self.app.post("/buckets/bid/collections/cid/search",
                                 headers=self.headers)
            result = resp.json
            assert result == {}

    def test_invalid_search_query(self):
        body = {"whatever": {"wrong": "bad"}}
        resp = self.app.post_json("/buckets/bid/collections/cid/search",
                                  body,
                                  headers=self.headers,
                                  status=400)
        assert resp.json["message"] == "Unknown key for a START_OBJECT in [whatever]."
        assert resp.json["details"] == {
            "col": 14,
            "line": 1,
            "reason": "Unknown key for a START_OBJECT in [whatever].",
            "type": "parsing_exception"
        }

    def test_search_on_empty_collection_returns_empty_list(self):
        resp = self.app.post("/buckets/bid/collections/cid/search",
                             headers=self.headers)
        result = resp.json
        assert len(result["hits"]["hits"]) == 0

    def test_querystring_search_is_supported(self):
        self.app.post_json("/buckets/bid/collections/cid/records",
                           {"data": {"age": 12}}, headers=self.headers)
        self.app.post_json("/buckets/bid/collections/cid/records",
                           {"data": {"age": 21}}, headers=self.headers)
        resp = self.app.get("/buckets/bid/collections/cid/search?q=age:<15",
                             headers=self.headers)
        result = resp.json
        assert len(result["hits"]["hits"]) == 1
        assert result["hits"]["hits"][0]["_source"]["age"] == 12

    def test_empty_querystring_returns_all_results(self):
        self.app.post_json("/buckets/bid/collections/cid/records",
                           {"data": {"age": 12}}, headers=self.headers)
        self.app.post_json("/buckets/bid/collections/cid/records",
                           {"data": {"age": 21}}, headers=self.headers)
        resp = self.app.get("/buckets/bid/collections/cid/search",
                             headers=self.headers)
        result = resp.json
        assert len(result["hits"]["hits"]) == 2


class LimitedResults(BaseWebTest, unittest.TestCase):
    def get_app(self, settings):
        app = self.make_app(settings=settings)
        app.put("/buckets/bid", headers=self.headers)
        app.put("/buckets/bid/collections/cid", headers=self.headers)
        requests = [{
            "method": "POST",
            "path": "/buckets/bid/collections/cid/records",
            "body": {"data": {"age": i}}
        } for i in range(5)]
        app.post_json("/batch", {"requests": requests}, headers=self.headers)
        return app

    def test_the_number_of_responses_is_limited_by_paginate_by_setting(self):
        app = self.get_app({"paginate_by": 2})
        resp = app.get("/buckets/bid/collections/cid/search", headers=self.headers)
        result = resp.json
        assert len(result["hits"]["hits"]) == 2

    def test_the_number_of_responses_is_limited_by_max_fetch_size_setting(self):
        app = self.get_app({"storage_max_fetch_size": 2})
        resp = app.get("/buckets/bid/collections/cid/search", headers=self.headers)
        result = resp.json
        assert len(result["hits"]["hits"]) == 2

    def test_the_number_of_responses_is_limited_by_smaller_limit(self):
        app = self.get_app({"paginate_by": 4, "storage_max_fetch_size": 2})
        resp = app.get("/buckets/bid/collections/cid/search", headers=self.headers)
        result = resp.json
        assert len(result["hits"]["hits"]) == 2

    def test_the_number_of_responses_is_limited_by_only_defined_limit(self):
        app = self.get_app({"paginate_by": 0, "storage_max_fetch_size": 2})
        resp = app.get("/buckets/bid/collections/cid/search", headers=self.headers)
        result = resp.json
        assert len(result["hits"]["hits"]) == 2


class PermissionsCheck(BaseWebTest, unittest.TestCase):
    def test_search_is_allowed_if_write_on_bucket(self):
        body = {"permissions": {"write": ["system.Everyone"]}}
        self.app.put_json("/buckets/bid", body, headers=self.headers)
        self.app.put("/buckets/bid/collections/cid", headers=self.headers)

        self.app.post("/buckets/bid/collections/cid/search", status=200)

    def test_search_is_allowed_if_read_on_bucket(self):
        body = {"permissions": {"read": ["system.Everyone"]}}
        self.app.put_json("/buckets/bid", body, headers=self.headers)
        self.app.put("/buckets/bid/collections/cid", headers=self.headers)

        self.app.post("/buckets/bid/collections/cid/search", status=200)

    def test_search_is_allowed_if_write_on_collection(self):
        self.app.put("/buckets/bid", headers=self.headers)
        body = {"permissions": {"write": ["system.Everyone"]}}
        self.app.put_json("/buckets/bid/collections/cid", body, headers=self.headers)

        self.app.post("/buckets/bid/collections/cid/search", status=200)

    def test_search_is_allowed_if_read_on_collection(self):
        self.app.put("/buckets/bid", headers=self.headers)
        body = {"permissions": {"read": ["system.Everyone"]}}
        self.app.put_json("/buckets/bid/collections/cid", body, headers=self.headers)

        self.app.post("/buckets/bid/collections/cid/search", status=200)

    def test_search_is_not_allowed_by_default(self):
        self.app.put("/buckets/bid", headers=self.headers)
        self.app.put("/buckets/bid/collections/cid", headers=self.headers)

        self.app.post("/buckets/bid/collections/cid/search", status=401)
        headers = get_user_headers("cual", "quiera")
        self.app.post("/buckets/bid/collections/cid/search", status=403, headers=headers)

    def test_search_is_not_allowed_if_only_read_on_certain_records(self):
        self.app.put("/buckets/bid", headers=self.headers)
        body = {"permissions": {"record:create": ["system.Authenticated"]}}
        self.app.put_json("/buckets/bid/collections/cid", body, headers=self.headers)
        headers = get_user_headers("toto")
        self.app.post_json("/buckets/bid/collections/cid/records", {"data": {"pi": 42}},
                           headers=headers)

        self.app.post("/buckets/bid/collections/cid/search", status=403, headers=headers)


class SchemaSupport(BaseWebTest, unittest.TestCase):

    schema = {
        "properties": {
            "id": {"type": "keyword"},
            "last_modified": {"type": "long"},
            "build": {
                "properties": {
                    "date": {"type": "date", "format": "strict_date"},
                    "id": {"type": "keyword"}
                }
            }
        }
    }

    def setUp(self):
        self.app.put("/buckets/bid", headers=self.headers)
        body = {"data": {"index:schema": self.schema}}
        self.app.put_json("/buckets/bid/collections/cid", body, headers=self.headers)
        self.app.post_json("/buckets/bid/collections/cid/records",
                           {"data": {"build": {"id": "abc", "date": "2017-05-24"}}},
                           headers=self.headers)
        self.app.post_json("/buckets/bid/collections/cid/records",
                           {"data": {"build": {"id": "efg", "date": "2017-02-01"}}},
                           headers=self.headers)

    def get_mapping(self, bucket_id, collection_id):
        indexer = self.app.app.registry.indexer
        indexname = indexer.indexname(bucket_id, collection_id)
        index_mapping = indexer.client.indices.get_mapping(indexname)
        mappings = index_mapping[indexname]["mappings"]
        return mappings.get(indexname, {})

    def test_index_has_mapping_if_collection_has_schema(self):
        mapping = self.get_mapping("bid", "cid")
        assert sorted(mapping["properties"].keys()) == ["build", "id", "last_modified"]

    def test_mapping_is_updated_on_collection_update(self):
        new_schema = copy.deepcopy(self.schema)
        new_schema["properties"]["build"]["properties"]["id"]["ignore_above"] = 12

        self.app.patch_json("/buckets/bid/collections/cid",
                            {"data": {"index:schema": new_schema}},
                            headers=self.headers)

        mapping = self.get_mapping("bid", "cid")
        assert mapping["properties"]["build"]["properties"]["id"]["ignore_above"] == 12

    def test_mapping_is_created_when_index_metadata_is_added(self):
        self.app.put("/buckets/bid/collections/cid2", headers=self.headers)
        mapping = self.get_mapping("bid", "cid2")
        assert "build" not in mapping.get("properties", {})

        self.app.patch_json("/buckets/bid/collections/cid2",
                            {"data": {"index:schema": self.schema}},
                            headers=self.headers)

        mapping = self.get_mapping("bid", "cid2")
        assert "build" in mapping["properties"]

    def test_mapping_is_preserved_when_index_metadata_is_removed(self):
        self.app.put_json("/buckets/bid/collections/cid",
                          {"data": {}},
                          headers=self.headers)

        mapping = self.get_mapping("bid", "cid")
        assert "build" in mapping["properties"]

    def test_can_search_for_subproperties(self):
        body = {
            "query": {
                "bool" : {
                    "must" : {
                        "term" : { "build.id" : "abc" }
                    }
                }
            }
        }
        resp = self.app.post_json("/buckets/bid/collections/cid/search", body,
                                  headers=self.headers)
        result = resp.json
        assert len(result["hits"]["hits"]) == 1
        assert result["hits"]["hits"][0]["_source"]["build"]["id"] == "abc"

    def test_can_aggregate_values(self):
        body = {
          "aggs" : {
            "build_dates" : {
              "terms": {
                "field" : "build.id",
                "size" : 1000
              }
            }
          }
        }
        resp = self.app.post_json("/buckets/bid/collections/cid/search", body,
                                  headers=self.headers)
        result = resp.json
        assert result["aggregations"]["build_dates"]["buckets"] == [
            {"key": "abc", "doc_count": 1},
            {"key": "efg", "doc_count": 1},
        ]

    def test_search_parse_exception_returns_400(self):
        e = elasticsearch.RequestError('', '',
                                       {"error": "Could not find aggregator type"})
        with mock.patch("kinto_elasticsearch.indexer.Indexer.search",
                        side_effect=e):
            self.app.post_json("/buckets/bid/collections/cid/search",
                               headers=self.headers, status=400)



import logging

import elasticsearch
from kinto.core.events import ACTIONS


logger = logging.getLogger(__name__)


def on_collection_created(event):
    indexer = event.request.registry.indexer
    bucket_id = event.payload["bucket_id"]
    for created in event.impacted_records:
        collection_id = created["new"]["id"]
        schema = created["new"].get("index:schema")
        indexer.create_index(bucket_id, collection_id, schema=schema)


def on_collection_updated(event):
    indexer = event.request.registry.indexer
    bucket_id = event.payload["bucket_id"]
    for updated in event.impacted_records:
        collection_id = updated["new"]["id"]
        old_schema = updated["old"].get("index:schema")
        new_schema = updated["new"].get("index:schema")
        # Create if there was no index before.
        if old_schema is None and new_schema is not None:
            indexer.create_index(bucket_id, collection_id, schema=new_schema)
        elif old_schema != new_schema:
            indexer.update_index(bucket_id, collection_id, schema=new_schema)


def on_collection_deleted(event):
    indexer = event.request.registry.indexer
    bucket_id = event.payload["bucket_id"]
    for deleted in event.impacted_records:
        collection_id = deleted["old"]["id"]
        indexer.delete_index(bucket_id, collection_id)


def on_bucket_deleted(event):
    indexer = event.request.registry.indexer
    for deleted in event.impacted_records:
        bucket_id = deleted["old"]["id"]
        indexer.delete_index(bucket_id)


def on_record_changed(event):
    indexer = event.request.registry.indexer

    bucket_id = event.payload["bucket_id"]
    collection_id = event.payload["collection_id"]
    action = event.payload["action"]

    try:
        with indexer.bulk() as bulk:
            for change in event.impacted_records:
                if action == ACTIONS.DELETE.value:
                    bulk.unindex_record(bucket_id,
                                        collection_id,
                                        record=change["old"],
                                        id_field="id")
                else:
                    bulk.index_record(bucket_id,
                                      collection_id,
                                      record=change["new"],
                                      id_field="id")
    except elasticsearch.ElasticsearchException:
        logger.exception("Failed to index record")


def on_server_flushed(event):
    indexer = event.request.registry.indexer
    indexer.flush()



import pkg_resources

from kinto.events import ServerFlushed
from kinto.core.events import AfterResourceChanged

from . import indexer
from . import listener


#: Module version, as defined in PEP-0396.
__version__ = pkg_resources.get_distribution(__package__).version


def includeme(config):
    # Register a global indexer object
    config.registry.indexer = indexer.load_from_config(config)

    # Register heartbeat to check elasticsearch integration.
    config.registry.heartbeats["elasticsearch"] = indexer.heartbeat

    # Activate end-points.
    config.scan("kinto_elasticsearch.views")

    config.add_subscriber(listener.on_record_changed, AfterResourceChanged,
                          for_resources=("record",))
    config.add_subscriber(listener.on_server_flushed, ServerFlushed)
    config.add_subscriber(listener.on_collection_created, AfterResourceChanged,
                          for_resources=("collection",), for_actions=("create",))
    config.add_subscriber(listener.on_collection_updated, AfterResourceChanged,
                          for_resources=("collection",), for_actions=("update",))
    config.add_subscriber(listener.on_collection_deleted, AfterResourceChanged,
                          for_resources=("collection",), for_actions=("delete",))
    config.add_subscriber(listener.on_bucket_deleted, AfterResourceChanged,
                          for_resources=("bucket",), for_actions=("delete",))

    config.add_api_capability("elasticsearch",
                              description="Index and search records using ElasticSearch.",
                              url="https://github.com/Kinto/kinto-elasticsearch",
                              version=__version__)



import logging

import elasticsearch
from kinto.core import authorization
from kinto.core import Service
from kinto.core import utils
from kinto.core.errors import http_error, ERRORS
from pyramid import httpexceptions


logger = logging.getLogger(__name__)


class RouteFactory(authorization.RouteFactory):
    def __init__(self, request):
        super().__init__(request)
        records_plural = utils.strip_uri_prefix(request.path.replace("/search", "/records"))
        self.permission_object_id = records_plural
        self.required_permission = "read"


search = Service(name="search",
                 path='/buckets/{bucket_id}/collections/{collection_id}/search',
                 description="Search",
                 factory=RouteFactory)


def search_view(request, **kwargs):
    bucket_id = request.matchdict['bucket_id']
    collection_id = request.matchdict['collection_id']

    # Limit the number of results to return, based on existing Kinto settings.
    paginate_by = request.registry.settings.get("paginate_by")
    max_fetch_size = request.registry.settings["storage_max_fetch_size"]
    if paginate_by is None or paginate_by <= 0:
        paginate_by = max_fetch_size
    kwargs.setdefault("size", min(paginate_by, max_fetch_size))

    # Access indexer from views using registry.
    indexer = request.registry.indexer
    try:
        results = indexer.search(bucket_id, collection_id, **kwargs)

    except elasticsearch.NotFoundError as e:
        # If plugin was enabled after the creation of the collection.
        indexer.create_index(bucket_id, collection_id)
        results = indexer.search(bucket_id, collection_id, **kwargs)

    except elasticsearch.RequestError as e:
        # Malformed query.
        if isinstance(e.info["error"], dict):
            message = e.info["error"]["reason"]
            details = e.info["error"]["root_cause"][0]
        else:
            message = e.info["error"]
            details = None
        response = http_error(httpexceptions.HTTPBadRequest(),
                              errno=ERRORS.INVALID_PARAMETERS,
                              message=message,
                              details=details)
        raise response

    except elasticsearch.ElasticsearchException as e:
        # General failure.
        logger.exception("Index query failed.")
        results = {}

    return results


@search.post(permission=authorization.DYNAMIC)
def post_search(request):
    body = request.body
    return search_view(request, body=body)


@search.get(permission=authorization.DYNAMIC)
def get_search(request):
    q = request.GET.get("q")
    return search_view(request, q=q)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from setuptools import setup


with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('CHANGELOG.rst') as history_file:
    history = history_file.read()

requirements = [
    'elasticsearch',
    'kinto>=6.0.0'
]

test_requirements = [
    'mock',
    'unittest2',
    'webtest',
]

setup(
    name='kinto-elasticsearch',
    version='0.2.0',
    description="Index and search records using ElasticSearch.",
    long_description=readme + '\n\n' + history,
    author='Mozilla Services',
    author_email='services-dev@mozilla.com',
    url='https://github.com/kinto/kinto-elasticsearch',
    packages=[
        'kinto_elasticsearch',
    ],
    package_dir={'kinto_elasticsearch': 'kinto_elasticsearch'},
    include_package_data=True,
    install_requires=requirements,
    license="Apache License (2.0)",
    zip_safe=False,
    keywords='kinto elasticsearch index',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    test_suite='tests',
    tests_require=test_requirements
)
